Package: MSCquartets
Type: Package
Title: Analyzing Gene Tree Quartets under the Multi-Species Coalescent
Version: 3.3
Authors@R: c(
     person("Elizabeth", "Allman", email = "esallman@alaska.edu",
                  role = "aut"),
     person("Hector", "Banos", email = "hbassnos@gmail.com",
                  role =  "aut"),
     person("Jonathan", "Mitchell", email = "jonathanmitchell88@gmail.com",
                  role =  "aut"),
     person("Kristina", "Wicke", email = "kristina.wicke@njit.edu",
                  role =  "aut"),              
     person("John", "Rhodes", email = "j.rhodes@alaska.edu",
                  role = c("aut", "cre"),comment=c(ORCID = "0000-0001-9921-1091"))  )
Description: Methods for analyzing and using quartets displayed on a collection 
 of gene trees, primarily to make inferences about the species tree or network 
 under the multi-species coalescent model. These include quartet hypothesis tests
 for the model, as developed by Mitchell et al. (2019) <doi:10.1214/19-EJS1576>, 
 simplex plots of quartet concordance factors as presented by Allman et al. (2020) 
 <doi:10.1101/2020.02.13.948083>, species tree inference methods based on quartet 
 distances of Rhodes (2019) <doi:10.1109/TCBB.2019.2917204> and Yourdkhani and 
 Rhodes (2019) <doi:10.1007/s11538-020-00773-4>, the NANUQ algorithm for inference 
 of level-1 species networks of Allman et al. (2019) <doi:10.1186/s13015-019-0159-2>, 
 the TINNIK algorithm for inference of the tree of blobs of an arbitrary 
 network of Allman et al.(2022) <doi:10.1007/s00285-022-01838-9>,
 NANUQ+ routines for resolving multifurcations in the tree of blobs to cycles as in
 Rhodes et al.(2024) <doi:10.1186/s13015-025-00274-w>, and the ECToBlob algorithm for inference of a 
 network with no anomalous quartets of Allman et al. (2026) (forthcoming).
 Software announcement by Rhodes et al. (2020) <doi:10.1093/bioinformatics/btaa868>. 
License: MIT + file LICENSE
Imports: zipfR, graphics, stats, Rdpack, foreach, doParallel, methods,
        Rcpp (>= 1.0.10), igraph (>= 2.0.0)
RdMacros: Rdpack
Encoding: UTF-8
LazyData: true
NeedsCompilation: yes
LinkingTo: Rcpp, RcppProgress
Depends: R (>= 3.5.0), ape (>= 5.0), phangorn
Packaged: 2026-05-27 15:50:22 UTC; jarhodes2
Author: Elizabeth Allman [aut],
  Hector Banos [aut],
  Jonathan Mitchell [aut],
  Kristina Wicke [aut],
  John Rhodes [aut, cre] (ORCID: <https://orcid.org/0000-0001-9921-1091>)
Maintainer: John Rhodes <j.rhodes@alaska.edu>
Suggests: knitr, rmarkdown
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
RoxygenNote: 7.3.3
Repository: CRAN
Date/Publication: 2026-05-28 07:20:02 UTC
