# Generated by roxygen2: do not edit by hand

export(BQinference)
export(ECToBlob)
export(ECToBlob_bi)
export(ECToBlob_mul)
export(ECToBlob_quad)
export(HolmBonferroni)
export(M0)
export(NANUQ)
export(NANUQdist)
export(QDC)
export(QDS)
export(T1density)
export(T3density)
export(TINNIK)
export(TINNIKdist)
export(WQDC)
export(WQDCrecursive)
export(WQDS)
export(WQDSAdjustLengths)
export(allPerms)
export(blobDistance)
export(circHybOrders)
export(collapseEdges)
export(combineCycleResolutions)
export(combineP)
export(compatibleSplits)
export(cutDensity)
export(estimateEdgeLengths)
export(expNANUQCycleDist)
export(expectedCFs)
export(expmodNANUQCycleDist)
export(fitCycleOrders)
export(initBquartets)
export(labelIntNodes)
export(nexusDist)
export(nodeGroups)
export(ordersHeuristicmodNANUQ)
export(powerDivStat)
export(pvalHist)
export(quartetBTinferencePlot)
export(quartetCutMLE)
export(quartetCutTest)
export(quartetCutTestInd)
export(quartetDist)
export(quartetDistTree)
export(quartetECtestInd)
export(quartetNetworkDist)
export(quartetStarTest)
export(quartetStarTestInd)
export(quartetTable)
export(quartetTableCollapse)
export(quartetTableDominant)
export(quartetTableParallel)
export(quartetTablePrint)
export(quartetTableResolved)
export(quartetTallyCpp)
export(quartetTestPlot)
export(quartetTreeErrorProb)
export(quartetTreeTest)
export(quartetTreeTestInd)
export(quartetWeightedDist)
export(resolveCycle)
export(resolveLevel1)
export(simplexCoords)
export(simplexLabels)
export(simplexPoint)
export(simplexPrepare)
export(simplexSegment)
export(simplexText)
export(sortQuartetTableRows)
export(suppressNetBinaryNodes)
export(taxonNames)
export(topDist)
export(treeFromSplits)
export(treeOfBlobs)
importFrom(Rcpp,evalCpp)
importFrom(Rcpp,sourceCpp)
importFrom(Rdpack,reprompt)
importFrom(ape,.PlotPhyloEnv)
importFrom(ape,Ntip)
importFrom(ape,bind.tree)
importFrom(ape,compute.brlen)
importFrom(ape,cophenetic.phylo)
importFrom(ape,di2multi)
importFrom(ape,drop.tip)
importFrom(ape,evonet)
importFrom(ape,fastme.bal)
importFrom(ape,is.rooted)
importFrom(ape,keep.tip)
importFrom(ape,nj)
importFrom(ape,nodelabels)
importFrom(ape,read.evonet)
importFrom(ape,read.tree)
importFrom(ape,root)
importFrom(ape,unroot)
importFrom(ape,write.evonet)
importFrom(doParallel,registerDoParallel)
importFrom(foreach,"%dopar%")
importFrom(foreach,foreach)
importFrom(graphics,hist)
importFrom(graphics,legend)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,segments)
importFrom(graphics,text)
importFrom(igraph,E)
importFrom(igraph,V)
importFrom(igraph,all_simple_paths)
importFrom(igraph,as.igraph)
importFrom(igraph,degree)
importFrom(igraph,difference)
importFrom(igraph,distances)
importFrom(igraph,edge_attr)
importFrom(igraph,intersection)
importFrom(igraph,is_directed)
importFrom(methods,is)
importFrom(phangorn,Descendants)
importFrom(phangorn,as.networx)
importFrom(phangorn,compatible)
importFrom(phangorn,mrca.phylo)
importFrom(stats,chisq.test)
importFrom(stats,cophenetic)
importFrom(stats,dmultinom)
importFrom(stats,integrate)
importFrom(stats,na.omit)
importFrom(stats,pbinom)
importFrom(stats,pcauchy)
importFrom(stats,pchisq)
importFrom(stats,pnorm)
importFrom(stats,qnorm)
importFrom(stats,rmultinom)
importFrom(stats,runif)
importFrom(stats,uniroot)
importFrom(stats,var)
importFrom(utils,combn)
importFrom(zipfR,Ibeta)
useDynLib(MSCquartets)
